@@ -57,6 +57,8 @@ typedef enum {
     PREF_INTYPE,
     PREF_HISTORY,
     PREF_MOUSE,
+    PREF_OCCUPANTS,
+    PREF_MUC_PRIVILEGES,
     PREF_STATUSES,
     PREF_STATUSES_CONSOLE,
     PREF_STATUSES_CHAT,
