@@ -82,8 +82,6 @@ prof_run(const int disable_tls, char *log_level, char *account_name)
     char inp[INP_WIN_MAX];
     int size = 0;
 
-    ui_update_screen();
-
     char *pref_connect_account = prefs_get_string(PREF_CONNECT_ACCOUNT);
     if (account_name != NULL) {
         char *cmd = "/connect";
@@ -95,9 +93,11 @@ prof_run(const int disable_tls, char *log_level, char *account_name)
         process_input(inp);
     }
     prefs_free_string(pref_connect_account);
+    ui_update();
 
     while(cmd_result == TRUE) {
         wint_t ch = ERR;
+        int result;
         size = 0;
 
         while(ch != '\n') {
@@ -114,14 +114,14 @@ prof_run(const int disable_tls, char *log_level, char *account_name)
                 g_timer_start(timer);
             }
 
-            ui_handle_special_keys(&ch, inp, size);
-            ui_update_screen();
+            ui_handle_special_keys(&ch, result, inp, size);
 #ifdef HAVE_LIBOTR
             otr_poll();
 #endif
             jabber_process_events();
+            ui_update();
 
-            ch = ui_get_char(inp, &size);
+            ch = ui_get_char(inp, &size, &result);
         }
 
         inp[size++] = '\0';
@@ -218,7 +218,6 @@ process_input(char *inp)
 
     ui_input_clear();
     roster_reset_search_attempts();
-    ui_current_page_off();
 
     return result;
 }
