@@ -120,7 +120,7 @@ inp_block(void)
 }
 
 wint_t
-inp_get_char(char *input, int *size)
+inp_get_char(char *input, int *size, int *result)
 {
     wint_t ch;
     int display_size = 0;
@@ -131,7 +131,7 @@ inp_get_char(char *input, int *size)
 
     // echo off, and get some more input
     noecho();
-    int result = wget_wch(inp_win, &ch);
+    *result = wget_wch(inp_win, &ch);
 
     gboolean in_command = FALSE;
     if ((display_size > 0 && input[0] == '/') ||
@@ -140,12 +140,12 @@ inp_get_char(char *input, int *size)
     }
 
     if (prefs_get_boolean(PREF_STATES)) {
-        if (result == ERR) {
+        if (*result == ERR) {
             prof_handle_idle();
         }
         if (prefs_get_boolean(PREF_OUTTYPE)
-                && (result != ERR)
-                && (result != KEY_CODE_YES)
+                && (*result != ERR)
+                && (*result != KEY_CODE_YES)
                 && !in_command
                 && _printable(ch)) {
             prof_handle_activity();
@@ -153,8 +153,8 @@ inp_get_char(char *input, int *size)
     }
 
     // if it wasn't an arrow key etc
-    if (!_handle_edit(result, ch, input, size)) {
-        if (_printable(ch) && result != KEY_CODE_YES) {
+    if (!_handle_edit(*result, ch, input, size)) {
+        if (_printable(ch) && *result != KEY_CODE_YES) {
             if (*size >= INP_WIN_MAX) {
                 return ERR;
             }
@@ -272,7 +272,7 @@ _clear_input(void)
 /*
  * Deal with command editing, return 1 if ch was an edit
  * key press: up, down, left, right or backspace
- * return 0 if it wasnt
+ * return 0 if it wasn't
  */
 static int
 _handle_edit(int result, const wint_t ch, char *input, int *size)
@@ -290,7 +290,7 @@ _handle_edit(int result, const wint_t ch, char *input, int *size)
     inp_x = getcurx(inp_win);
 
     // CTRL-LEFT
-    if ((result == KEY_CODE_YES) && (ch == 547 || ch == 545 || ch == 540 || ch == 539) && (inp_x > 0)) {
+    if ((result == KEY_CODE_YES) && (ch == 547 || ch == 545 || ch == 544 || ch == 540 || ch == 539) && (inp_x > 0)) {
         input[*size] = '\0';
         gchar *curr_ch = g_utf8_offset_to_pointer(input, inp_x);
         curr_ch = g_utf8_find_prev_char(input, curr_ch);
@@ -340,7 +340,7 @@ _handle_edit(int result, const wint_t ch, char *input, int *size)
         return 1;
 
     // CTRL-RIGHT
-    } else if ((result == KEY_CODE_YES) && (ch == 562 || ch == 560 || ch == 555 || ch == 554) && (inp_x < display_size)) {
+    } else if ((result == KEY_CODE_YES) && (ch == 562 || ch == 560 || ch == 555 || ch == 559 || ch == 554) && (inp_x < display_size)) {
         input[*size] = '\0';
         gchar *curr_ch = g_utf8_offset_to_pointer(input, inp_x);
         gchar *next_ch = g_utf8_find_next_char(curr_ch, NULL);
